# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc
import warnings

from . import inference_pb2 as inference__pb2

GRPC_GENERATED_VERSION = "1.78.0"
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower

    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f"The grpc package installed is at version {GRPC_VERSION},"
        + " but the generated code in inference_pb2_grpc.py depends on"
        + f" grpcio>={GRPC_GENERATED_VERSION}."
        + f" Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}"
        + f" or downgrade your generated code using grpcio-tools<={GRPC_VERSION}."
    )


class InferenceStub(object):
    """Inference service provides cluster inference configuration and bundle delivery."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetInferenceBundle = channel.unary_unary(
            "/openshell.inference.v1.Inference/GetInferenceBundle",
            request_serializer=inference__pb2.GetInferenceBundleRequest.SerializeToString,
            response_deserializer=inference__pb2.GetInferenceBundleResponse.FromString,
            _registered_method=True,
        )
        self.SetClusterInference = channel.unary_unary(
            "/openshell.inference.v1.Inference/SetClusterInference",
            request_serializer=inference__pb2.SetClusterInferenceRequest.SerializeToString,
            response_deserializer=inference__pb2.SetClusterInferenceResponse.FromString,
            _registered_method=True,
        )
        self.GetClusterInference = channel.unary_unary(
            "/openshell.inference.v1.Inference/GetClusterInference",
            request_serializer=inference__pb2.GetClusterInferenceRequest.SerializeToString,
            response_deserializer=inference__pb2.GetClusterInferenceResponse.FromString,
            _registered_method=True,
        )


class InferenceServicer(object):
    """Inference service provides cluster inference configuration and bundle delivery."""

    def GetInferenceBundle(self, request, context):
        """Return the resolved inference route bundle for sandbox-local execution."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SetClusterInference(self, request, context):
        """Set cluster-level inference configuration.

        This controls how requests sent to `inference.local` are routed.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetClusterInference(self, request, context):
        """Get cluster-level inference configuration."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_InferenceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "GetInferenceBundle": grpc.unary_unary_rpc_method_handler(
            servicer.GetInferenceBundle,
            request_deserializer=inference__pb2.GetInferenceBundleRequest.FromString,
            response_serializer=inference__pb2.GetInferenceBundleResponse.SerializeToString,
        ),
        "SetClusterInference": grpc.unary_unary_rpc_method_handler(
            servicer.SetClusterInference,
            request_deserializer=inference__pb2.SetClusterInferenceRequest.FromString,
            response_serializer=inference__pb2.SetClusterInferenceResponse.SerializeToString,
        ),
        "GetClusterInference": grpc.unary_unary_rpc_method_handler(
            servicer.GetClusterInference,
            request_deserializer=inference__pb2.GetClusterInferenceRequest.FromString,
            response_serializer=inference__pb2.GetClusterInferenceResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "openshell.inference.v1.Inference", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers("openshell.inference.v1.Inference", rpc_method_handlers)


# This class is part of an EXPERIMENTAL API.
class Inference(object):
    """Inference service provides cluster inference configuration and bundle delivery."""

    @staticmethod
    def GetInferenceBundle(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/openshell.inference.v1.Inference/GetInferenceBundle",
            inference__pb2.GetInferenceBundleRequest.SerializeToString,
            inference__pb2.GetInferenceBundleResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def SetClusterInference(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/openshell.inference.v1.Inference/SetClusterInference",
            inference__pb2.SetClusterInferenceRequest.SerializeToString,
            inference__pb2.SetClusterInferenceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def GetClusterInference(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/openshell.inference.v1.Inference/GetClusterInference",
            inference__pb2.GetClusterInferenceRequest.SerializeToString,
            inference__pb2.GetClusterInferenceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )



"""Protobuf ↔ dict conversion helpers shared across client managers."""

from __future__ import annotations

from ._proto import sandbox_pb2


def _dict_to_policy(data: dict) -> sandbox_pb2.SandboxPolicy:
    """Convert a policy dict to protobuf SandboxPolicy.

    Args:
        data: Policy definition with optional keys ``version``,
            ``filesystem``/``filesystem_policy``, ``process``,
            ``landlock``, and ``network_policies``.

    Returns:
        sandbox_pb2.SandboxPolicy: Populated protobuf message.
    """
    policy = sandbox_pb2.SandboxPolicy()
    if "version" in data:
        policy.version = data["version"]
    if "filesystem" in data or "filesystem_policy" in data:
        fs = data.get("filesystem") or data.get("filesystem_policy", {})
        policy.filesystem.CopyFrom(
            sandbox_pb2.FilesystemPolicy(
                include_workdir=fs.get("include_workdir", False),
                read_only=fs.get("read_only", []),
                read_write=fs.get("read_write", []),
            )
        )
    if "process" in data:
        policy.process.CopyFrom(
            sandbox_pb2.ProcessPolicy(
                run_as_user=data["process"].get("run_as_user", ""),
                run_as_group=data["process"].get("run_as_group", ""),
            )
        )
    if "landlock" in data:
        policy.landlock.CopyFrom(
            sandbox_pb2.LandlockPolicy(
                compatibility=data["landlock"].get("compatibility", ""),
            )
        )
    if "network_policies" in data:
        for key, rule_data in data["network_policies"].items():
            policy.network_policies[key].CopyFrom(_dict_to_network_rule(rule_data))
    return policy


def _dict_to_l7_query(query_data: dict) -> dict[str, sandbox_pb2.L7QueryMatcher]:
    """Convert a query matcher dict to protobuf L7QueryMatcher map.

    Args:
        query_data: Mapping of parameter names to matcher definitions
            with ``glob`` (str) and/or ``any`` (list[str]) keys.

    Returns:
        dict[str, sandbox_pb2.L7QueryMatcher]: Protobuf matcher map.
    """
    result: dict[str, sandbox_pb2.L7QueryMatcher] = {}
    for key, matcher in query_data.items():
        result[key] = sandbox_pb2.L7QueryMatcher(
            glob=matcher.get("glob", ""),
            **{"any": matcher.get("any", [])},
        )
    return result


def _dict_to_network_rule(data: dict) -> sandbox_pb2.NetworkPolicyRule:
    """Convert a network rule dict to protobuf.

    Args:
        data: Network rule definition with ``name``, ``endpoints``,
            and ``binaries`` keys.

    Returns:
        sandbox_pb2.NetworkPolicyRule: Populated protobuf message.
    """
    rule = sandbox_pb2.NetworkPolicyRule(name=data.get("name", ""))
    for ep_data in data.get("endpoints", []):
        ep = sandbox_pb2.NetworkEndpoint(
            host=ep_data.get("host", ""),
            port=ep_data.get("port", 0),
            protocol=ep_data.get("protocol", ""),
            tls=ep_data.get("tls", ""),
            enforcement=ep_data.get("enforcement", ""),
            access=ep_data.get("access", ""),
            allowed_ips=ep_data.get("allowed_ips", []),
            ports=ep_data.get("ports", []),
        )
        for rule_data in ep_data.get("rules", []):
            allow = rule_data.get("allow", {})
            l7_allow = sandbox_pb2.L7Allow(
                method=allow.get("method", ""),
                path=allow.get("path", ""),
                command=allow.get("command", ""),
            )
            if "query" in allow:
                for k, v in _dict_to_l7_query(allow["query"]).items():
                    l7_allow.query[k].CopyFrom(v)
            ep.rules.append(sandbox_pb2.L7Rule(allow=l7_allow))
        rule.endpoints.append(ep)
    for bin_data in data.get("binaries", []):
        rule.binaries.append(sandbox_pb2.NetworkBinary(path=bin_data.get("path", "")))
    return rule



"""Provider management with metadata from openshell.yaml."""

from __future__ import annotations

from typing import Any

from shoreguard.client import ShoreGuardClient
from shoreguard.services._openshell_meta import get_openshell_meta


class ProviderService:
    """Provider management shared by Web UI and TUI.

    Args:
        client: OpenShell gRPC client instance.
    """

    def __init__(self, client: ShoreGuardClient) -> None:  # noqa: D107
        self._client = client

    def list(self, *, limit: int = 100, offset: int = 0) -> list[dict[str, Any]]:
        """List all providers.

        Args:
            limit: Maximum number of providers to return.
            offset: Number of providers to skip.

        Returns:
            list[dict[str, Any]]: Provider records.
        """
        return self._client.providers.list(limit=limit, offset=offset)

    def get(self, name: str) -> dict[str, Any]:
        """Get a provider by name.

        Args:
            name: Provider name.

        Returns:
            dict[str, Any]: Provider record.
        """
        return self._client.providers.get(name)

    def create(
        self,
        *,
        name: str,
        provider_type: str,
        api_key: str,
        extra_credentials: dict[str, str] | None = None,
        config: dict[str, str] | None = None,
    ) -> dict[str, Any]:
        """Create a provider with automatic credential key mapping.

        Looks up the correct environment variable name from openshell.yaml.
        Falls back to API_KEY for unknown types.

        Args:
            name: Provider name.
            provider_type: Provider type identifier.
            api_key: Primary API key value.
            extra_credentials: Additional credential key-value pairs.
            config: Optional provider configuration.

        Returns:
            dict[str, Any]: The created provider record.
        """
        meta = get_openshell_meta()
        type_info = meta.get_provider_type(provider_type)
        cred_key = type_info.get("cred_key", "API_KEY") if type_info else "API_KEY"

        credentials = {cred_key: api_key}
        if extra_credentials:
            credentials.update(extra_credentials)

        return self._client.providers.create(
            name=name,
            provider_type=provider_type,
            credentials=credentials,
            config=config,
        )

    def update(
        self,
        *,
        name: str,
        provider_type: str = "",
        credentials: dict[str, str] | None = None,
        config: dict[str, str] | None = None,
    ) -> dict[str, Any]:
        """Update an existing provider.

        Args:
            name: Provider name.
            provider_type: New provider type (empty string to keep current).
            credentials: New credential key-value pairs.
            config: New provider configuration.

        Returns:
            dict[str, Any]: The updated provider record.
        """
        return self._client.providers.update(
            name=name,
            provider_type=provider_type,
            credentials=credentials,
            config=config,
        )

    def delete(self, name: str) -> bool:
        """Delete a provider.

        Args:
            name: Provider name.

        Returns:
            bool: True if the provider was deleted.
        """
        return self._client.providers.delete(name)

    @staticmethod
    def list_known_types() -> list[dict[str, str]]:
        """Return metadata about known provider types from openshell.yaml.

        Returns:
            list[dict[str, str]]: Provider type metadata records.
        """
        meta = get_openshell_meta()
        return [{"type": k, **v} for k, v in meta.provider_types.items()]

    @staticmethod
    def list_inference_providers() -> list[dict[str, str]]:
        """Return known inference provider options from openshell.yaml.

        Returns:
            list[dict[str, str]]: Inference provider option records.
        """
        meta = get_openshell_meta()
        return meta.inference_providers

    @staticmethod
    def list_community_sandboxes() -> list[dict[str, Any]]:
        """Return community sandbox templates from openshell.yaml.

        Returns:
            list[dict[str, Any]]: Community sandbox template records.
        """
        meta = get_openshell_meta()
        return meta.community_sandboxes



"""Draft policy approval flow service."""

from __future__ import annotations

from typing import Any

from shoreguard.client import ShoreGuardClient


class ApprovalService:
    """Draft policy approval operations shared by Web UI and TUI.

    Args:
        client: OpenShell gRPC client instance.
    """

    def __init__(self, client: ShoreGuardClient) -> None:  # noqa: D107
        self._client = client

    def get_draft(self, sandbox_name: str, *, status_filter: str = "") -> dict[str, Any]:
        """Get draft policy recommendations for a sandbox.

        Args:
            sandbox_name: Name of the sandbox.
            status_filter: Optional status to filter by.

        Returns:
            dict[str, Any]: Draft policy data.
        """
        return self._client.approvals.get_draft(sandbox_name, status_filter=status_filter)

    def get_pending(self, sandbox_name: str) -> list[dict[str, Any]]:
        """Get only pending (unapproved) draft chunks.

        Args:
            sandbox_name: Name of the sandbox.

        Returns:
            list[dict[str, Any]]: Pending draft chunks.
        """
        return self._client.approvals.get_pending(sandbox_name)

    def approve(self, sandbox_name: str, chunk_id: str) -> dict[str, Any]:
        """Approve a single draft policy chunk.

        Args:
            sandbox_name: Name of the sandbox.
            chunk_id: Identifier of the chunk to approve.

        Returns:
            dict[str, Any]: Updated chunk data.
        """
        return self._client.approvals.approve(sandbox_name, chunk_id)

    def reject(self, sandbox_name: str, chunk_id: str, *, reason: str = "") -> None:
        """Reject a single draft policy chunk.

        Args:
            sandbox_name: Name of the sandbox.
            chunk_id: Identifier of the chunk to reject.
            reason: Optional reason for rejection.
        """
        return self._client.approvals.reject(sandbox_name, chunk_id, reason=reason)

    def approve_all(
        self, sandbox_name: str, *, include_security_flagged: bool = False
    ) -> dict[str, Any]:
        """Approve all pending draft chunks.

        Args:
            sandbox_name: Name of the sandbox.
            include_security_flagged: Whether to include security-flagged chunks.

        Returns:
            dict[str, Any]: Summary of approved chunks.
        """
        return self._client.approvals.approve_all(
            sandbox_name, include_security_flagged=include_security_flagged
        )

    def edit(self, sandbox_name: str, chunk_id: str, proposed_rule: dict) -> None:
        """Edit a pending draft chunk's proposed rule.

        Args:
            sandbox_name: Name of the sandbox.
            chunk_id: Identifier of the chunk to edit.
            proposed_rule: New proposed rule content.
        """
        return self._client.approvals.edit(sandbox_name, chunk_id, proposed_rule)

    def undo(self, sandbox_name: str, chunk_id: str) -> dict[str, Any]:
        """Reverse an approval decision.

        Args:
            sandbox_name: Name of the sandbox.
            chunk_id: Identifier of the chunk to undo.

        Returns:
            dict[str, Any]: Updated chunk data.
        """
        return self._client.approvals.undo(sandbox_name, chunk_id)

    def clear(self, sandbox_name: str) -> dict[str, int]:
        """Clear all pending draft chunks for a sandbox.

        Args:
            sandbox_name: Name of the sandbox.

        Returns:
            dict[str, int]: Count of cleared chunks.
        """
        return self._client.approvals.clear(sandbox_name)

    def get_history(self, sandbox_name: str) -> list[dict[str, Any]]:
        """Get decision history for a sandbox's draft policy.

        Args:
            sandbox_name: Name of the sandbox.

        Returns:
            list[dict[str, Any]]: Decision history entries.
        """
        return self._client.approvals.get_history(sandbox_name)
